@@ -1,6 +1,6 @@
 import asyncio
 import logging
-from typing import Dict, Callable, List, Optional, Union, Coroutine
+from typing import Dict, Callable, List, Optional, Union, Coroutine, IO
 
 from .. import AsyncRunnable, MessageTypes, EventTypes  # interfaces & basics
 from .. import Cert, HTTPRequester, WebhookReceiver, WebsocketReceiver, Gateway, Client  # net related
@@ -164,9 +164,9 @@ class Bot(AsyncRunnable):
         """fetch details of a public channel from khl"""
         return await self.client.fetch_public_channel(channel_id)
 
-    async def delete_channel(self, channel_id: str):
+    async def delete_channel(self, channel: Union[Channel, str]):
         """delete a channel, permission required"""
-        return await self.client.delete_channel(channel_id)
+        return await self.client.delete_channel(channel)
 
     async def fetch_guild(self, guild_id: str) -> Guild:
         """fetch details of a guild from khl"""
@@ -179,7 +179,12 @@ class Bot(AsyncRunnable):
         return await self.client.list_guild()
 
     @staticmethod
-    async def send(target: Channel, content: Union[str, List], *, temp_target_id: str = '', **kwargs):
+    async def send(target: Channel,
+                   content: Union[str, List],
+                   *,
+                   type: MessageTypes = None,
+                   temp_target_id: str = '',
+                   **kwargs):
         """
         send a msg to a channel
 
@@ -188,14 +193,21 @@ class Bot(AsyncRunnable):
         if isinstance(target, PublicTextChannel):
             kwargs['temp_target_id'] = temp_target_id
 
-        return await target.send(content, **kwargs)
+        return await target.send(content, type=type, **kwargs)
 
-    async def upload_asset(self, file: str) -> str:
-        """upload ``file`` to khl, and return the url to the file, alias for ``create_asset``"""
-        return await self.client.create_asset(file)
+    async def upload_asset(self, file: Union[IO, str]) -> str:
+        """DEPRECATED, will be removed in a future release: use ``create_asset()`` instead
 
-    async def create_asset(self, file: str) -> str:
-        """upload ``file`` to khl, and return the url to the file"""
+        upload ``file`` to khl, and return the url to the file, alias for ``create_asset``
+
+        if ``file`` is a str, ``open(file, 'rb')`` will be called to convert it into IO"""
+        log.info('CAUTION: Bot.upload_asset() is DEPRECATED, please use create_asset() instead')
+        return await self.create_asset(file)
+
+    async def create_asset(self, file: Union[IO, str]) -> str:
+        """upload ``file`` to khl, and return the url to the file
+
+        if ``file`` is a str, ``open(file, 'rb')`` will be called to convert it into IO"""
         return await self.client.create_asset(file)
 
     async def kickout(self, guild: Guild, user: Union[User, str]):
@@ -210,43 +222,27 @@ class Bot(AsyncRunnable):
             raise ValueError('can not modify guild from other gate')
         return await guild.leave()
 
-    async def _make_temp_msg(self, msg_id: str) -> Message:
-        return Message(msg_id=msg_id, _gate_=self.client.gate)
-
-    async def delete_message(self, msg: Union[Message, str]):
-        """delete msg
-
-        wraps `Message.delete`
-
-        :param msg: the msg, accepts Message and msg_id(str)
-        """
-        if isinstance(msg, str):
-            msg = self._make_temp_msg(msg)
-        return await msg.delete()
-
-    async def add_reaction(self, msg: Union[Message, str], emoji: str):
+    @staticmethod
+    async def add_reaction(msg: Message, emoji: str):
         """add emoji to msg's reaction list
 
         wraps `Message.add_reaction`
 
-        :param msg: accepts `Message` and msg_id(str)
+        :param msg: accepts `Message`
         :param emoji: 😘
         """
-        if isinstance(msg, str):
-            msg = self._make_temp_msg(msg)
         return await msg.add_reaction(emoji)
 
-    async def delete_reaction(self, msg: Union[Message, str], emoji: str, user: User = None):
+    @staticmethod
+    async def delete_reaction(msg: Message, emoji: str, user: User = None):
         """delete emoji from msg's reaction list
 
         wraps `Message.delete_reaction`
 
-        :param msg: accepts `Message` and msg_id(str)
+        :param msg: accepts `Message`
         :param emoji: 😘
         :param user: whose reaction, delete others added reaction requires channel msg admin permission
         """
-        if isinstance(msg, str):
-            msg = self._make_temp_msg(msg)
         return await msg.delete_reaction(emoji, user)
 
     async def start(self):
